// ADPersonPropertyView+Private.m (this is -*- ObjC -*-)
// 
// \author: Bjrn Giesler <giesler@ira.uka.de>
// 
// Address Book Application for GNUstep
// 
// $Author: giesler $
// $Locker:  $
// $Revision: 1.3 $
// $Date: 2002/08/19 08:41:28 $

/* system includes */
/* (none) */

/* my includes */
#include "ADPersonPropertyView+Private.h"

@implementation ADPersonPropertyView (Private)
+ (NSString*) defaultLabelForProperty: (NSString*) property
{
  NSDictionary *dict =
    [NSDictionary dictionaryWithObjectsAndKeys:
		    ADEmailWorkLabel, ADEmailProperty,
		  ADAddressWorkLabel, ADAddressProperty,
		  ADPhoneWorkLabel, ADPhoneProperty,
		  ADAIMWorkLabel, ADAIMInstantProperty,
		  ADJabberWorkLabel, ADJabberInstantProperty,
		  ADMSNWorkLabel, ADMSNInstantProperty,
		  ADYahooWorkLabel, ADYahooInstantProperty,
		  ADICQWorkLabel, ADICQInstantProperty,
		  nil];
  NSString *str = [dict objectForKey: property];
  if(str) return str;
  return ADWorkLabel;
}

- (ADPersonPropertyCell*) addCellWithValue: (NSString*) val
				    inRect: (NSRect*) rect
				  editable: (BOOL) yesno
				      font: (NSFont*) font
				 alignment: (NSTextAlignment) alignment
				   details: (id) details
{
  ADPersonPropertyCell *cell;

  cell = [[[ADPersonPropertyCell alloc] init] autorelease];
  [cell setEditable: yesno];
  [cell setStringValue: val];
  [cell setFont: font];
  [cell setAlignment: alignment];

  if(details) [cell setDetails: details];

  NSSize s = [cell cellSize];
  rect->size.width = MAX(rect->size.width, s.width);
  rect->size.height = MAX(rect->size.height, s.height);

  [cell setRect: *rect];
  [_cells addObject: cell];

  return cell;
}
  
- (ADPersonPropertyCell*) addValueCellForValue: (NSString*) val
					inRect: (NSRect*) rect
				       details: (id) details
{
  ADPersonPropertyCell *cell = [self addCellWithValue: val
				     inRect: rect
				     editable: YES
				     font: [self font]
				     alignment: NSLeftTextAlignment
				     details: details];
  
  if(_delegate &&
     [_delegate respondsToSelector: _clickSel] &&
     [_delegate canPerformClickForProperty: _property] &&
     !_editable)
    [cell setTextColor: [NSColor blueColor]];
  else if(_editable)
    [cell setTextColor: [NSColor darkGrayColor]];
  else
    [cell setTextColor: [NSColor blackColor]];
  return cell;
}

- (ADPersonPropertyCell*) addValueCellForValue: (NSString*) val
					inRect: (NSRect*) rect
{
  return [self addValueCellForValue: val
	       inRect: rect
	       details: nil];
}

- (ADPersonPropertyCell*) addLabelCellForLabel: (NSString*) label
					inRect: (NSRect*) rect
{
  NSString *str = ADLocalizedPropertyOrLabel(label);
  NSFont *font = [self boldFont];
  int w = [font widthOfString: str];
  _neededLabelWidth = MAX(_neededLabelWidth, w);
  str = [str stringByAbbreviatingToFitWidth: [self maxLabelWidth]
	     inFont: font];
  
  rect->size.width = [self maxLabelWidth];
  
  return [self addCellWithValue: str
	       inRect: rect
	       editable: NO
	       font: font
	       alignment: NSRightTextAlignment
	       details: nil];
}

- (void) layoutForEdit
{
  if(!_property || !_person) return;

  id val = [_person valueForProperty: _property];
  id cell;

  ADPropertyType type = [[_person class] typeOfProperty: _property];

  NSRect r; NSSize allSize; 
  r = NSMakeRect(0, 0, 0, 0); allSize = NSMakeSize(0, 0);
  int i;

  // convert types
  if(val)
    switch(type)
      {
      case ADStringProperty: break;
      case ADIntegerProperty:
	val = [NSString stringWithFormat: @"%d", [val intValue]];
	break;
      case ADRealProperty:
	val = [NSString stringWithFormat: @"%f", [val floatValue]];
	break;
      case ADDateProperty:
      case ADArrayProperty:
      case ADDictionaryProperty:
      case ADDataProperty:
      case ADErrorInProperty:
	NSLog(@"Can't layout object of type %d\n", type);
	return;
      default:
	break;
      }

  // layout single-cell values
  if(!(type & ADMultiValueMask))
    {
      if(_displaysLabel)
	{
	  [self addLabelCellForLabel: _property inRect: &r];
	  allSize.width += r.size.width + 5;
	  allSize.height = r.size.height;
	  r.origin.x += r.size.width + 5; r.size = NSMakeSize(0, 0);
	}

      if(!val)
	{
	  val = [NSString stringWithFormat: @"[%@]",
			  ADLocalizedPropertyOrLabel(_property)];
	  cell = [self addValueCellForValue: val
		       inRect: &r];
	  [cell setTextColor: [NSColor darkGrayColor]];
	}
      else
	cell = [self addValueCellForValue: val
		     inRect: &r];
      allSize.width += r.size.width;
      allSize.height = MAX(allSize.height, r.size.height);
    }

  // layout multi-cell values; multi-string only this far
  else if(type == ADMultiStringProperty)
    {
      if(![val count])
	{
	  ADPersonPropertyCell *labelCell, *valueCell;
	  NSString *label, *value;
	  NSDictionary *details;

	  label = [[self class] defaultLabelForProperty: _property];

	  details = [NSDictionary dictionaryWithObjectsAndKeys:
				    label, @"Label",
				  nil];
	  
	  labelCell = [self addLabelCellForLabel: label inRect: &r];
	  [labelCell setDetails: details];
	  r.origin.x += r.size.width + 5; 
	  allSize.width = r.size.width + 5;
	  allSize.height = r.size.height;
	  r.size = NSMakeSize(0, 0);

	  value = [NSString stringWithFormat: @"[%@]",
			    ADLocalizedPropertyOrLabel(_property)];
	  valueCell = [self addValueCellForValue: value inRect: &r];
	  [valueCell setTextColor: [NSColor darkGrayColor]];
	  [valueCell setDetails: details];

	  allSize.width += r.size.width;
	  allSize.height = MAX(allSize.height, r.size.height);
	}
      else
	{
	  for(i=0; i<[val count]; i++)
	    {
	      ADPersonPropertyCell *labelCell, *valueCell;
	      NSString *label, *identifier, *value;
	      NSDictionary *details;
	      NSSize rowSize;
	      
	      label = [val labelAtIndex: i];
	      identifier = [val identifierAtIndex: i];
	      value = [val valueAtIndex: i];
	      
	      details = [NSDictionary dictionaryWithObjectsAndKeys:
					identifier, @"Identifier",
				      label, @"Label",
				      nil];
	      
	      labelCell = [self addLabelCellForLabel: label inRect: &r];
	      [labelCell setDetails: details];
	      r.origin.x += r.size.width + 5; 
	      rowSize.width = r.size.width + 5;
	      rowSize.height = r.size.height;
	      r.size = NSMakeSize(0, 0);
	      
	      valueCell = [self addValueCellForValue: value inRect: &r];
	      [valueCell setDetails: details];
	      [valueCell setTextColor: [NSColor darkGrayColor]];
	      
	      rowSize.width += r.size.width;
	      rowSize.height = MAX(rowSize.height, r.size.height);
	      r.size = NSMakeSize(0, 0);
	      r.origin.x = 0;
	      r.origin.y += rowSize.height;
	      
	      allSize.width = MAX(allSize.width, rowSize.width);
	      allSize.height += rowSize.height;
	    }
	  // FIXME: Add "new property" button here
	}
    }

  else
    NSLog(@"Can't layout values of type %d yet\n", type);

  _requiredSize = allSize;
  [self setFrameSize: _requiredSize];
}


- (void) layoutForDisplay
{
  if(!_property || !_person) return;

  id val = [_person valueForProperty: _property];
  id cell;

  ADPropertyType type = [[_person class] typeOfProperty: _property];

  NSRect r; NSSize allSize; 
  r = NSMakeRect(0, 0, 0, 0); allSize = NSMakeSize(0, 0);
  int i;

  if(!val) return;

  // convert types
  switch(type)
    {
    case ADStringProperty: break;
    case ADIntegerProperty:
      val = [NSString stringWithFormat: @"%d", [val intValue]];
      break;
    case ADRealProperty:
      val = [NSString stringWithFormat: @"%f", [val floatValue]];
      break;
    case ADDateProperty:
    case ADArrayProperty:
    case ADDictionaryProperty:
    case ADDataProperty:
    case ADErrorInProperty:
      NSLog(@"Can't layout object of type %d\n", type);
      return;
    default:
      break;
    }

  // layout single-cell values
  if(!(type & ADMultiValueMask))
    {
      if(_displaysLabel)
	{
	  [self addLabelCellForLabel: _property inRect: &r];
	  allSize.width += r.size.width + 5;
	  allSize.height = r.size.height;
	  r.origin.x += r.size.width + 5; r.size = NSMakeSize(0, 0);
	}

      cell = [self addValueCellForValue: val inRect: &r];
      allSize.width += r.size.width;
      allSize.height = MAX(allSize.height, r.size.height);
    }

  // layout multi-cell values; multi-string only this far
  else if(type == ADMultiStringProperty)
    {
      for(i=0; i<[val count]; i++)
	{
	  ADPersonPropertyCell *labelCell, *valueCell;
	  NSString *label, *identifier, *value;
	  NSSize rowSize;

	  label = [val labelAtIndex: i];
	  identifier = [val identifierAtIndex: i];
	  value = [val valueAtIndex: i];
	  
	  labelCell = [self addLabelCellForLabel: label inRect: &r];
	  r.origin.x += r.size.width + 5; 
	  rowSize.width = r.size.width + 5;
	  rowSize.height = r.size.height;
	  r.size = NSMakeSize(0, 0);
	      
	  valueCell = [self addValueCellForValue: value inRect: &r];
	      
	  rowSize.width += r.size.width;
	  rowSize.height = MAX(rowSize.height, r.size.height);
	  r.size = NSMakeSize(0, 0);
	  r.origin.x = 0;
	  r.origin.y += rowSize.height;
	  
	  allSize.width = MAX(allSize.width, rowSize.width);
	  allSize.height += rowSize.height;
	}
    }
  else
    NSLog(@"Can't layout values of type %d yet\n", type);

  _requiredSize = allSize;
  [self setFrameSize: _requiredSize];
}
- (void) layout
{
  _neededLabelWidth = 0;
  
  // clear everything
  [_cells release]; _cells = [[NSMutableArray alloc] init];

  if([self isEditable])
    [self layoutForEdit];
  else
    [self layoutForDisplay];
}
@end

