// ADPersonPropertyView.h (this is -*- ObjC -*-)
// 
// \author: Bjrn Giesler <giesler@ira.uka.de>
// 
// Address Book Application for GNUstep
// 
// $Author: giesler $
// $Locker:  $
// $Revision: 1.3 $
// $Date: 2002/08/19 08:41:28 $

#ifndef ADPERSONPROPERTYVIEW_H
#define ADPERSONPROPERTYVIEW_H

/* system includes */
#include <AppKit/AppKit.h>
#include <AddressBook/AddressBook.h>

/* my includes */
/* (none) */

@interface NSString(ADPersonPropertySupport)
- (NSString*) stringByAbbreviatingToFitWidth: (int) width
				      inFont: (NSFont*) font;
- (NSString*) stringByTrimmingWhitespace;
- (BOOL) isEmptyString;
@end

@interface ADPersonPropertyCell: NSTextFieldCell
{
  NSRect _r;
  id _details;
}
- (void) setRect: (NSRect) r;
- (NSRect) rect;

- (void) setDetails: (id) details;
- (id) details;
@end

@interface ADPersonPropertyView: NSView
{
  ADPerson *_person;
  NSString *_property;
  NSMutableArray *_cells; 
  BOOL _displaysLabel;
  int _maxLabelWidth, _neededLabelWidth;
  int _fontSize;
  NSFont *_font; BOOL _fontSetExternally;

  BOOL _editable;
  int _editingCellIndex; id _textObject;
  id _delegate;

  SEL _clickSel, _changeSel, _canPerformSel, _widthSel, _editInNextSel;

  NSSize _requiredSize;
}

- (void) setDelegate: (id) delegate;
- (id) delegate;

- (void) setPerson: (ABPerson*) person;
- (ABPerson*) person;

- (void) setProperty: (NSString*) property;
- (NSString*) property;

- (void) setDisplaysLabel: (BOOL) yesno;
- (BOOL) displaysLabel;

- (void) setMaxLabelWidth: (int) width;
- (int) maxLabelWidth;
- (int) neededLabelWidth;

- (NSFont*) font;
- (void) setFont: (NSFont*) font;
- (NSFont*) boldFont;
- (int) fontSize;
- (void) setFontSize: (int) size;

- (void) setEditable: (BOOL) editable;
- (BOOL) isEditable;

- (BOOL) hasEditableCells;
- (void) beginEditingInCellAtIndex: (int) index;
- (void) endEditing;
@end

@protocol ADPersonPropertyViewDelegate
- (void) viewWillBeginEditing: (id) view; // must implement this
- (BOOL) canPerformClickForProperty: (id) property;
- (void) clickedOnProperty: (id) value
		 withValue: (id) property
		    inView: (id) sender;
- (void) valueForProperty: (id) property
	   changedToValue: (id) value
		   inView: (id) sender;
- (void) view: (id) view
changedWidthFrom: (float) oldW
	   to: (float) newW;
- (void) beginEditingInNextViewWithTextMovement: (int) textMovement;
@end

#endif /* ADPERSONPROPERTYVIEW_H */
