// ADPersonView.m (this is -*- ObjC -*-)
// 
// \author: Bjrn Giesler <giesler@ira.uka.de>
// 
// 
// 
// $Author: bjoern $
// $Locker:  $
// $Revision: 1.1.1.1 $
// $Date: 2003/08/28 15:54:43 $

#include "ADPersonView.h"
#include "ADPersonPropertyView.h"

@interface ADImageView: NSImageView
{
  id _target;
  SEL _selector;
}
- (void) setTarget: (id) target;
- (void) setAction: (SEL) sel;
- (void) mouseDown: (NSEvent*) event;
- (BOOL) hasEditableCells;
@end

@implementation ADImageView
- (void) setTarget: (id) target
{
  _target = target;
}

- (void) setAction: (SEL) sel
{
  _selector = sel;
}

- (void) mouseDown: (NSEvent*) event
{
  if([_target respondsToSelector: _selector])
    [_target performSelector: _selector withObject: self];
}

- (BOOL) hasEditableCells
{
  return NO;
}
@end

@implementation ADPersonView
- initWithFrame: (NSRect) frameRect
{
  [super initWithFrame: frameRect];

  _person = nil;
  _editable = NO;

  [[NSNotificationCenter defaultCenter]
    addObserver: self
    selector: @selector(superviewFrameChanged:)
    name: NSViewFrameDidChangeNotification
    object: nil];
  return self;
}

- (void) dealloc
{
  [labels release];
  [firstNameCell release];
  [lastNameCell release];
  [titleCell release];
  [photoCell release];
  [_person release];

  [allCells release];

  [[NSNotificationCenter defaultCenter] removeObserver: self];
}

- (BOOL) isFlipped
{
  return YES;
}

- (int) layoutHeaderAndReturnNextY
{
  ADImageView *iv;
  ADPersonPropertyView *v;
  NSSize sizeNeeded;

#define IMGWIDTH 64.0
#define IMGHEIGHT 50.0
  int x = 5;
  int y = 5;

  // Image
  iv = [[ADImageView alloc]
	 initWithFrame: NSMakeRect(x, y, IMGWIDTH, IMGHEIGHT)];
  [self addSubview: iv];
  [iv setTarget: self];
  [iv setAction: @selector(imageClicked:)];
  
  NSString *imgPath = [_person imageDataFile];
  if(!imgPath) imgPath = [[NSBundle bundleForClass: [self class]]
			   pathForImageResource: @"UnknownImage.tiff"];
  if(!imgPath) NSLog(@"Error: UnknownImage.tiff not found!\n");
  else
    {
      NSImage *img = [[[NSImage alloc] initWithContentsOfFile: imgPath]
		       autorelease];
      if(!img) NSLog(@"Error: Couldn't load %@\n", imgPath);
      else
	[iv setImage: img];
    }

  x += IMGWIDTH + 10;
  
  
  // First name
  v = [[ADPersonPropertyView alloc] initWithFrame: NSMakeRect(x, y, 0, 0)];
  [v setEditable: _editable];
  [v setDelegate: self];
  [v setFontSize: 20];
  [v setFont: [v boldFont]];
  [v setPerson: _person];
  [v setProperty: ADFirstNameProperty];
  [self addSubview: v];
  sizeNeeded = [v frame].size;
  sizeNeeded.width += [[v font] widthOfString: @"f"];
  sizeNeeded.height += 5;
  
  // Last name
  v = [[ADPersonPropertyView alloc]
	initWithFrame: NSMakeRect(x+sizeNeeded.width, y, 0, 0)];
  [v setEditable: _editable];
  [v setDelegate: self];
  [v setFontSize: 20];
  [v setFont: [v boldFont]];
  [v setPerson: _person];
  [v setProperty: ADLastNameProperty];
  [self addSubview: v];
  sizeNeeded.width += [v frame].size.width;
  sizeNeeded.height = MAX(sizeNeeded.height, [v frame].size.height);

  y = sizeNeeded.height;
  v = [[ADPersonPropertyView alloc] initWithFrame: NSMakeRect(x, y, 0, 0)];
  [v setEditable: _editable];
  [v setDelegate: self];
  [v setPerson: _person];
  [v setProperty: ADOrganizationProperty];
  [self addSubview: v];
  if([v frame].size.height)
    sizeNeeded.height += [v frame].size.height;

  // Job title
  y = sizeNeeded.height;
  v = [[ADPersonPropertyView alloc] initWithFrame: NSMakeRect(x, y, 0, 0)];
  [v setEditable: _editable];
  [v setDelegate: self];
  [v setPerson: _person];
  [v setProperty: ADJobTitleProperty];
  [self addSubview: v];
  if([v frame].size.height)
    sizeNeeded.height += [v frame].size.height;

  if(sizeNeeded.height > IMGHEIGHT)
    {
      NSRect imgRect = [iv frame];
      // +3: compensate for the space down to the line. Looks better.
      imgRect.origin.y = (sizeNeeded.height-imgRect.size.height)/2+3;
      [iv setFrameOrigin: imgRect.origin];
    }
  
  return MAX(sizeNeeded.height, IMGHEIGHT+10);
}

- (void) layout
{
  NSArray *properties = [NSArray arrayWithObjects: ADHomePageProperty,
				 ADPhoneProperty,
				 ADEmailProperty,
				 ADAddressProperty,
				 nil];
			     
  if(!_person) return;

  while([[self subviews] count])
    [[[self subviews] objectAtIndex: 0] removeFromSuperview];

  int y = [self layoutHeaderAndReturnNextY];

  _headerLineY = y + 7;
  y += 15;

  NSEnumerator *e;
  NSString *property;

  e = [properties objectEnumerator];
  while((property = [e nextObject]))
    {
      ADPersonPropertyView *v;
      
      v = [[ADPersonPropertyView alloc] initWithFrame: NSMakeRect(5, y, 0, 0)];
      [v setEditable: _editable];
      [v setDelegate: self];
      [v setDisplaysLabel: YES];
      [v setPerson: _person];
      [v setProperty: property];
      [self addSubview: v];

      if([v frame].size.height)
	y += [v frame].size.height + 15;
    }

  _footerLineY = y - 8;

  [self calcSize];
}

- (BOOL) fillsSuperview
{
  return _fillsSuperview;
}

- (void) setFillsSuperview: (BOOL) yesno
{
  _fillsSuperview = yesno;
  [self calcSize];
}

- (void) calcSize
{
  NSEnumerator *e;
  
  NSSize sizeNeeded = NSMakeSize(0, 0);
  e = [[self subviews] objectEnumerator];
  NSView *view;
  while((view = [e nextObject]))
    {
      NSRect r = [view frame];

      sizeNeeded.height = r.origin.y + r.size.height;
      sizeNeeded.width = MAX(sizeNeeded.width,
			     r.origin.x + r.size.width);
    }
  sizeNeeded.width += 10;

  if(_fillsSuperview && [self superview])
    {
      NSSize superSize = [[self superview] frame].size;
      if(sizeNeeded.height < superSize.height)
	sizeNeeded.height = superSize.height;
      if(sizeNeeded.width < superSize.width)
	sizeNeeded.width = superSize.width;
    }

  [self setFrameSize: sizeNeeded];
}

- (void) setPerson: (ADPerson*) aPerson
{
  if(aPerson == _person) return;
  
  [_person release];
  _person = [aPerson retain];

  [allCells release];
  allCells = nil;

  selfResized = YES;
  [self setFrame: NSZeroRect];
  [self layout];
}

- (ADPerson*) person
{
  return _person;
}

- (void) setDelegate: (id) anObject
{
  delegate = anObject;
}

- (id) delegate
{
  return delegate;
}

- (void) drawRect: (NSRect) rect
{
  [super drawRect: rect];

   if(!_person) return;
  
  [self lockFocus];

  if(![self isEditable])
    [[NSColor whiteColor] set];
  else
    [[NSColor colorWithDeviceRed: 1.0 green: 1.0 blue: 0.5 alpha: 1.0] set];
  NSRectFill(rect);

  [[NSColor blackColor] set];
  NSBezierPath *p;
  p = [NSBezierPath bezierPath];
  [p moveToPoint: NSMakePoint(5, _headerLineY)];
  [p lineToPoint: NSMakePoint([self frame].size.width-5, _headerLineY)];
  [p stroke];
  p = [NSBezierPath bezierPath];
  [p moveToPoint: NSMakePoint(5, _footerLineY)];
  [p lineToPoint: NSMakePoint([self frame].size.width-5, _footerLineY)];
  [p stroke];
  
  [self unlockFocus];
}

- (BOOL) isEditable
{
  return _editable;
}

- (void) setEditable: (BOOL) yn
{
  if(yn == _editable)
    return;
  _editable = yn;
  [self layout];
}

- (void) superviewFrameChanged: (NSNotification*) note
{
  if([self isDescendantOf: [note object]] && self != [note object])
    [self layout];
}

- (void) imageClicked: (id) sender
{
  if(!_editable) return;

  NSOpenPanel *panel = [NSOpenPanel openPanel];
  NSArray *types = [NSArray arrayWithObjects: @"jpg", @"JPG", @"jpeg", @"JPEG",
			    @"tiff", @"TIFF", @"tif", @"TIF", nil];
  [panel setCanChooseFiles: YES];
  [panel setCanChooseDirectories: NO];
  [panel setAllowsMultipleSelection: NO];
  int retval = [panel runModalForTypes: types];

  if(retval == NSCancelButton) return;

  if([[panel filenames] count] != 1)
    {
      NSLog(@"Argh! %d filenames; 1 expected\n", [[panel filenames] count]);
      return;
    }
  if(![_person setImageDataWithFile: [[panel filenames] objectAtIndex: 0]])
    NSRunAlertPanel(_(@"Error Loading Image"),
		    [NSString stringWithFormat: _(@"The image file %@ could "
						  @"not be loaded.")],
		    _(@"OK"), nil, nil, nil);
  else
    [self layout];
}
  
/*
 * Delegate methods
 */

- (BOOL) canPerformClickForProperty: (id) property
{
  if([property isEqualToString: ADEmailProperty] ||
     [property isEqualToString: ADHomePageProperty])
    return YES;
  return NO;
}

- (void) clickedOnProperty: (id) property
		 withValue: (id) value
		    inView: (id) sender
{
  if([property isEqualToString: ADEmailProperty])
    {
      NSPasteboard *pb = [NSPasteboard generalPasteboard];
      [pb declareTypes: [NSArray arrayWithObjects: NSStringPboardType, nil]
	  owner: self];
      [pb setString: value forType: NSStringPboardType];

      NSPerformService(_(@"GNUMail/New Mail with recipient"), pb);
    }
  else if([property isEqualToString: ADHomePageProperty])
    {
      NSPasteboard *pb = [NSPasteboard generalPasteboard];
      [pb declareTypes: [NSArray arrayWithObjects: NSStringPboardType, nil]
	  owner: self];
      [pb setString: value forType: NSStringPboardType];

      NSPerformService(@"Open URL", pb);
    }
}

- (void) valueForProperty: (id) property
	   changedToValue: (id) value
		   inView: (id) sender
{
  [self layout];
}

- (void) viewWillBeginEditing: (id) view
{
  int i;

  for(i=0; i<[[self subviews] count]; i++)
    {
      id v = [[self subviews] objectAtIndex: i];
      if(v == view)
	_editingViewIndex = i;
      else if([v isKindOfClass: [ADPersonPropertyView class]])
	[v endEditing];
    }
}

- (void) view: (id) view
changedWidthFrom: (float) w1
	   to: (float) w2
{
  NSPoint o = [view frame].origin;
  NSEnumerator *e = [[self subviews] objectEnumerator];
  ADPersonPropertyView *v;

  while((v = [e nextObject]))
    {
      if(v == view) continue;
      NSPoint p = [v frame].origin;
      if(p.y == o.y && p.x > o.x)
	{
	  p.x += (w2-w1);
	  [v setFrameOrigin: p];
	}
    }
  [self setNeedsDisplay: YES];
}

- (void) beginEditingInNextViewWithTextMovement: (int) movement
{
  NSArray *subs = [self subviews];
  if(![subs count]) return;
  
  switch(movement)
    {
    case NSReturnTextMovement:
      return;
      
    case NSTabTextMovement:
      do {
	_editingViewIndex++;
	if(_editingViewIndex >= [subs count])
	  _editingViewIndex = 0;
      } while(![[subs objectAtIndex: _editingViewIndex] hasEditableCells]);
      break;
    case NSBacktabTextMovement:
      do {
	_editingViewIndex--;
	if(_editingViewIndex < 0)
	  _editingViewIndex = [subs count]-1;
      } while(![[subs objectAtIndex: _editingViewIndex] hasEditableCells]);
      break;
    default:
      break;
    }

  [[subs objectAtIndex: _editingViewIndex] beginEditingInCellAtIndex: 0];
}

/*
 * action methods
 */

- (void) mouseDown: (NSEvent*) event
{
  NSEnumerator *e = [[self subviews] objectEnumerator];
  id v;
  while((v = [e nextObject]))
    if([v isKindOfClass: [ADPersonPropertyView class]])
      [v endEditing];
  [self layout];
  [super mouseDown: event];
}
@end
