// ADSinglePropertyView.m (this is -*- ObjC -*-)
// 
// \author: Bjrn Giesler <giesler@ira.uka.de>
// 
// Address View Framework for GNUstep
// 
// $Author: giesler $
// $Locker:  $
// $Revision: 1.3 $
// $Date: 2002/08/19 08:41:28 $

/* system includes */
/* (none) */

/* my includes */
#include "ADSinglePropertyView.h"

// redefine _(@"...") so that it looks into our bundle, not the main bundle
#undef _
#define _(x) [[NSBundle bundleForClass: [ADSinglePropertyView class]] \
		   localizedStringForKey: x \
		   value: x \
		   table: nil]

@interface ADPerson (ScreenName)
- (NSString*) screenName;
- (int) compareByScreenName: (ADPerson*) theOtherGuy;
@end
@implementation ADPerson (ScreenName)
- (NSString*) screenName
{
  NSString *last, *first;
  last = [self valueForProperty: ADLastNameProperty];
  first = [self valueForProperty: ADFirstNameProperty];
  if(!last && !first)
    return @"New Person";
  else if(!first) return last;
  else if(!last) return first;
  else
    return [NSString stringWithFormat: @"%@, %@", last, first];
}

- (int) compareByScreenName: (ADPerson*) theOtherGuy
{
  if([[self screenName] isEqualToString: @"New Person"])
    return -1;
  else if([[theOtherGuy screenName] isEqualToString: @"New Person"])
    return 1;
  return [[self screenName] compare: [theOtherGuy screenName]];
}
@end

@interface ADSinglePropertyView (Private)
- (void) _buildArrays;
- (void) _handleDatabaseChanged: (NSNotification*) note;
- (void) _selectGroupInBrowser: (id) sender;
@end

@implementation ADSinglePropertyView (Private)
- (void) _buildArrays
{
  [_names release]; [_values release]; [_namesUnthinned release];
  _names = [[NSMutableArray alloc] init];
  _namesUnthinned = [[NSMutableArray alloc] init];
  _values = [[NSMutableArray alloc] init];

  ADPropertyType type = [[ADPerson class] typeOfProperty: _property];

  NSArray *people;
  if(!_selectedGroup)
    people = [_book people];
  else
    people = [_selectedGroup members];
  people = [people sortedArrayUsingSelector: @selector(compareByScreenName:)];

  NSEnumerator *e = [people objectEnumerator];
  ADPerson *p;
  while((p = [e nextObject]))
    {
      if(type & ADMultiValueMask)
	{
	  id val = [p valueForProperty: _property];
	  if(![val count]) continue;
	  int i;
	  
	  for(i=0; i<[val count]; i++)
	    {
	      if(i==0) [_names addObject: [p screenName]];
	      else [_names addObject: @""];
	      [_namesUnthinned addObject: [p screenName]];
	      [_values addObject: [[val valueAtIndex: i] description]];
	    }
	}
      else
	{
	  if(![p valueForProperty: _property]) continue;
	  [_names addObject: [p screenName]];
	  [_namesUnthinned addObject: [p screenName]];
	  [_values addObject: [[p valueForProperty: _property] description]];
	}
    }

  [_peopleTable reloadData];
  [_groupsBrowser reloadColumn: 0];
}

- (void) _handleDatabaseChanged: (NSNotification*) note
{
  [self _buildArrays];
}

- (void) _selectGroupInBrowser: (id) sender
{
  int row = [sender selectedRowInColumn: 0];

  if(!_book) _book = [ADAddressBook sharedAddressBook];
  [_selectedGroup release];
  if(row)
    _selectedGroup = [[[_book groups] objectAtIndex: row-1] retain];
  else
    _selectedGroup = nil;
  [self _buildArrays];
}
@end

@implementation ADSinglePropertyView
- initWithFrame: (NSRect) frame
{
  NSRect r;

  if(![super initWithFrame: frame]) return nil;

  [self setDisplayedProperty: ADEmailProperty];
  _selectedGroup = nil;
  
  [self setAutoresizesSubviews: YES];

  r = frame; r.origin = NSMakePoint(0, 0);
  _splitView = [[[NSSplitView alloc] initWithFrame: r] autorelease];
  [_splitView setAutoresizingMask: NSViewWidthSizable|NSViewHeightSizable];
  [_splitView setVertical: YES];
  [self addSubview: _splitView];

  r = frame; r.origin = NSMakePoint(0, 0);
  r.size.width = frame.size.width/4;
  _groupsBrowser = [[[NSBrowser alloc] initWithFrame: r] autorelease];
  [_groupsBrowser setMaxVisibleColumns: 1];
  [_groupsBrowser setDelegate: self];
  [_groupsBrowser setAction: @selector(_selectGroupInBrowser:)];
  [_groupsBrowser setTarget: self];
  [_groupsBrowser loadColumnZero];
  [_groupsBrowser selectRow: 0 inColumn: 0];
  [_splitView addSubview: _groupsBrowser];
  
  r = frame; r.origin = NSMakePoint(0, 0);
  r.size.width = (frame.size.width*3)/4;
  _ptScrollView = [[[NSScrollView alloc] initWithFrame: r] autorelease];
  [_ptScrollView setRulersVisible: YES];
  [_ptScrollView setHasVerticalScroller: YES];
  [_ptScrollView setHasHorizontalScroller: YES];
  [_ptScrollView setBorderType: NSBezelBorder];
  [_ptScrollView setAutoresizingMask: NSViewWidthSizable|NSViewHeightSizable];
  [_splitView addSubview: _ptScrollView];

  _peopleTable = [[[NSTableView alloc] initWithFrame: frame] autorelease];
  [_peopleTable setDataSource: self];

  _nameColumn = [[[NSTableColumn alloc] initWithIdentifier: @"Name"]
		  autorelease];
  [[_nameColumn headerCell] setStringValue: _(@"Person Name")];
  
  _propertyColumn = [[[NSTableColumn alloc] initWithIdentifier: @"Property"]
		      autorelease];
  [[_propertyColumn headerCell]
    setStringValue: ADLocalizedPropertyOrLabel(_property)];
  
  [_peopleTable addTableColumn: _nameColumn];
  [_peopleTable addTableColumn: _propertyColumn];
  [_peopleTable setAutoresizesAllColumnsToFit: YES];
  [_peopleTable setAllowsMultipleSelection: YES];
  [_peopleTable sizeToFit];

  [_ptScrollView setDocumentView: _peopleTable];

  [[NSNotificationCenter defaultCenter]
    addObserver: self
    selector: @selector(_handleDatabaseChanged:)
    name: ADDatabaseChangedExternallyNotification
    object: nil];
  
  return self;
}

- (void) dealloc
{
  [[NSNotificationCenter defaultCenter] removeObserver: self];
}

- (void) setDisplayedProperty: (NSString*) property
{
  if([_property isEqualToString: property]) return;
  if(!_book)
    _book = [ADAddressBook sharedAddressBook];
  
  ADPropertyType type = [[ADPerson class] typeOfProperty: property];
  if(type == ADErrorInProperty)
    {
      NSLog(@"Trying to set unknown property %@\n", property);
      return;
    }

  [_property release];
  _property = [property copy];

  [self _buildArrays];
  [[_propertyColumn headerCell]
    setStringValue: ADLocalizedPropertyOrLabel(_property)];
  [[_peopleTable headerView] setNeedsDisplay: YES];
  [_peopleTable deselectAll: self];
  [_peopleTable reloadData];
}

- (NSString*) displayedProperty
{
  return _property;
}

- (NSArray*) selectedValues
{
  NSMutableArray *retval = [NSMutableArray array];
  NSEnumerator *e = [_peopleTable selectedRowEnumerator];
  NSNumber *r;
  while((r = [e nextObject]))
    [retval addObject: [_values objectAtIndex: [r intValue]]];
  return [NSArray arrayWithArray: retval];
}

- (NSArray*) selectedNamesAndValues
{
  NSMutableArray *retval = [NSMutableArray array];
  NSEnumerator *e = [_peopleTable selectedRowEnumerator];
  NSNumber *r;
  while((r = [e nextObject]))
    {
      int i = [r intValue];
      [retval addObject: [NSArray arrayWithObjects: [_namesUnthinned objectAtIndex: i],
				  [_values objectAtIndex: i], nil]];
    }
  return [NSArray arrayWithArray: retval];  
}

/*
 * NSTableDataSource methods
 */

- (int) numberOfRowsInTableView: (NSTableView*) view
{
  return [_values count];
}

- (id) tableView: (NSTableView*) v
objectValueForTableColumn: (NSTableColumn*) col
	     row: (int) row
{
  NSString *val;
  if(col == _nameColumn)
    val = [_names objectAtIndex: row];
  else
    val = [_values objectAtIndex: row];
  return val;
}

/*
 * NSBrowser delegate methods
 */

- (int) browser: (NSBrowser*) b
numberOfRowsInColumn: (int) col
{
  if(!_book) _book = [ADAddressBook sharedAddressBook];
  return [[_book groups] count]+1;
}

- (NSString*) browser: (NSBrowser*) b
	titleOfColumn: (int) col
{
  return _(@"Groups");
}

- (void) browser: (NSBrowser*) b
 willDisplayCell: (NSBrowserCell*) cell
	   atRow: (int) row
	  column: (int) col
{
  if(!_book) _book = [ADAddressBook sharedAddressBook];
  if(row)
    [cell setStringValue: [[[_book groups] objectAtIndex: row-1]
			    valueForProperty: ADGroupNameProperty]];
  else
    [cell setStringValue: _(@"All")];
  [cell setLeaf: YES];
}
@end
