// ADEnvelopeAddressBook.m (this is -*- ObjC -*-)
// 
// \author: Bjrn Giesler <giesler@ira.uka.de>
// 
// Address Book Framework for GNUstep
// 
// $Author: giesler $
// $Locker:  $
// $Revision: 1.3 $
// $Date: 2002/08/19 08:41:28 $

/* system includes */
/* (none) */

/* my includes */
#include "ADLocalAddressBook.h"
#include "ADEnvelopeAddressBook.h"
#include "ADRecord.h"
#include "ADPerson.h"

static ADEnvelopeAddressBook *_envelopeAB = nil;

@implementation ADEnvelopeAddressBook
+ (ADAddressBook*) sharedAddressBook
{
  if(!_envelopeAB)
    {
      _envelopeAB =
	[[self alloc]
	  initWithPrimaryAddressBook: [ADLocalAddressBook sharedAddressBook]];
    }
  return _envelopeAB;
}

- initWithPrimaryAddressBook: (ADAddressBook*) book
{
  _merge = YES;
  _books = [[NSMutableArray alloc] initWithCapacity: 1];
  [self setPrimaryAddressBook: book];
  return self;
}

- (void) dealloc
{
  [_books release];
}

- (BOOL) addAddressBook: (ADAddressBook*) book
{
  if([_books indexOfObject: book] != NSNotFound)
    return NO;
  [_books addObject: book];
  return YES;
}

- (BOOL) removeAddressBook: (ADAddressBook*) book
{
  if([_books indexOfObject: book] == NSNotFound ||
     book == _primary)
    return NO;
  [_books removeObject: book];
  return YES;
}

- (void) setPrimaryAddressBook: (ADAddressBook*) book
{
  NSAssert(book, @"Primary address book cannot be nil");

  if([_books indexOfObject: book] == NSNotFound)
    [self addAddressBook: book];
  _primary = book;
}

- (ADAddressBook*) primaryAddressBook
{
  return _primary;
}

- (void) setMergesAddressBooks: (BOOL) merge
{
  _merge = merge;
}

- (BOOL) mergesAddressBooks
{
  return _merge;
}

/*
 * Subclass stuff
 */

- (NSArray*) recordsMatchingSearchElement: (ADSearchElement*) search
{
  NSMutableArray *arr = [NSMutableArray array];
  NSEnumerator *e = [_books objectEnumerator];
  ADAddressBook *book;
  while((book = [e nextObject]))
    [arr addObjectsFromArray: [book recordsMatchingSearchElement: search]];
  return [NSArray arrayWithArray: arr];
}

- (BOOL) save
{
  return [_primary save];
}

- (BOOL) hasUnsavedChanges
{
  return [_primary hasUnsavedChanges];
}

- (ADPerson*) me
{
  if(!_merge || [_primary me])  return [_primary me];

  NSEnumerator *e = [_books objectEnumerator];
  ADAddressBook *book;
  while((book = [e nextObject]))
    if([book me]) return [book me];
  return nil;
}
  
- (void) setMe: (ADPerson*) me
{
  return [[me addressBook] setMe: me];
}

- (ADRecord*) recordForUniqueId: (NSString*) uniqueId
{
  NSEnumerator *e = [_books objectEnumerator];
  ADAddressBook *book; ADRecord *retval;
  while((book = [e nextObject]))
    {
      retval = [book recordForUniqueId: uniqueId];
      if(retval) return retval;
    }

  return nil;
}

- (BOOL) addRecord: (ADRecord*) record
{
  return [_primary addRecord: record];
}

- (BOOL) removeRecord: (ADRecord*) record
{
  [[record addressBook] removeRecord: record];
  return YES;
}

- (NSArray*) people
{
  if(!_merge)
    return [_primary people];

  NSMutableArray *arr = [NSMutableArray arrayWithCapacity: 20];
  NSEnumerator *e = [_books objectEnumerator];
  ADAddressBook *book;

  while((book = [e nextObject]))
    [arr addObjectsFromArray: [book people]];
  return arr;
}

- (NSArray*) groups
{
  if(!_merge)
    return [_primary groups];

  NSMutableArray *arr = [NSMutableArray arrayWithCapacity: 20];
  NSEnumerator *e = [_books objectEnumerator];
  ADAddressBook *book;

  while((book = [e nextObject]))
    [arr addObjectsFromArray: [book groups]];
  return arr;
}

@end // ADEnvelopeAddressBook

@implementation ADEnvelopeAddressBook(GroupAccess)
- (NSArray*) membersForGroup: (ADGroup*) group
{
  [NSException raise: ADUnimplementedError
	       format: @"ADEnvelopeAddressBook cannot implement %@",
	       NSStringFromSelector(_cmd)];
  return nil;
}
- (BOOL) addMember: (ADPerson*) person forGroup: (ADGroup*) group
{ 
  [NSException raise: ADUnimplementedError
	       format: @"ADEnvelopeAddressBook cannot implement %@",
	       NSStringFromSelector(_cmd)];
  return NO;
}
- (BOOL) removeMember: (ADPerson*) person forGroup: (ADGroup*) group
{ 
  [NSException raise: ADUnimplementedError
	       format: @"ADEnvelopeAddressBook cannot implement %@",
	       NSStringFromSelector(_cmd)];
  return NO;
}

- (NSArray*) subgroupsForGroup: (ADGroup*) group
{ 
  [NSException raise: ADUnimplementedError
	       format: @"ADEnvelopeAddressBook cannot implement %@",
	       NSStringFromSelector(_cmd)];
  return nil;
}
- (BOOL) addSubgroup: (ADGroup*) g1 forGroup: (ADGroup*) g2
{ 
  [NSException raise: ADUnimplementedError
	       format: @"ADEnvelopeAddressBook cannot implement %@",
	       NSStringFromSelector(_cmd)];
  return NO;
}
- (BOOL) removeSubgroup: (ADGroup*) g1 forGroup: (ADGroup*) g2
{ 
  [NSException raise: ADUnimplementedError
	       format: @"ADEnvelopeAddressBook cannot implement %@",
	       NSStringFromSelector(_cmd)];
  return NO;
}
- (NSArray*) parentGroupsForGroup: (ADGroup*) group;
{ 
  [NSException raise: ADUnimplementedError
	       format: @"ADEnvelopeAddressBook cannot implement %@",
	       NSStringFromSelector(_cmd)];
  return nil;
}
@end
