// ADPListConverter.m (this is -*- ObjC -*-)
// 
// \author: Bjrn Giesler <giesler@ira.uka.de>
// 
// Address Book Framework for GNUstep
// 
// $Author: giesler $
// $Locker:  $
// $Revision: 1.3 $
// $Date: 2002/08/19 08:41:28 $

/* system includes */
/* (none) */

/* my includes */
#include "ADPListConverter.h"
#include "ADPerson.h"
#include "ADGroup.h"
#include "ADLocalAddressBook.h"
#include "ADMultiValue.h"

@implementation ADPListConverter
- initForInput
{
  _done = NO; _plist = nil;
  return [super init];
}

- (BOOL) useData: (NSData*) data
{
  char *bytes = calloc([data length]+1, 1);
  memcpy(bytes, [data bytes], [data length]);
  _plist = [[NSString stringWithCString: bytes] propertyList];
  if(![_plist isKindOf: [NSDictionary class]])
    {
      NSLog(@"Data (%s) does not contain valid property list!\n", bytes);
      return NO;
    }
  return YES;
}

- (ADRecord*) nextRecord
{
  NSMutableArray *keys = [NSMutableArray arrayWithArray: [_plist allKeys]];
  NSString *key;
  ADRecord *r;
  
  if([[_plist objectForKey: @"Type"] isEqualToString: @"Group"])
    {
      r = [[[ADGroup alloc] init] autorelease];
      NSArray *members = [_plist objectForKey: @"Members"];
      if(members)
	{
	  [r->_dict setObject: [NSMutableArray arrayWithArray: members]
	            forKey: @"Members"];
	  [keys removeObject: @"Members"];
	}
      else
	[r->_dict setObject: [NSMutableArray array] forKey: @"Members"];
    }
  else
     r = [[[ADPerson alloc] init] autorelease];
  
  NSEnumerator *e = [keys objectEnumerator];
  while((key = [e nextObject]))
    {
      id val = [_plist objectForKey: key];
      ADPropertyType t = [ADPerson typeOfProperty: key];
      if(t & ADMultiValueMask)
	{
	  ADMutableMultiValue *mv = [[[ADMutableMultiValue alloc]
				       initWithType: t]
				      autorelease];
	  int i;
	  for(i=0; i<[val count]; i++)
	    [mv->_arr addObject: [val objectAtIndex: i]];
	  [r->_dict setObject: mv forKey: key];
	}
      else
	{
	  switch(t)
	    {
	    case ADDateProperty:
	      [r->_dict setObject: [NSDate dateWithString: val] forKey: key];
	      break;
	    default:
	      [r->_dict setObject: val forKey: key];
	    }
	}
    }

  return r;
}
@end
