// ADRecord.m (this is -*- ObjC -*-)
// 
// \author: Bjrn Giesler <giesler@ira.uka.de>
// 
// Address Book Framework for GNUstep
// 
// $Author: giesler $
// $Locker:  $
// $Revision: 1.3 $
// $Date: 2002/08/19 08:41:28 $

/* system includes */
/* (none) */

/* my includes */
#include "ADAddressBook.h"
#include "ADRecord.h"
#include "ADGlobals.h"
#include "ADConverter.h"
#include "ADMultiValue.h"

@interface ADRecord(Private)
- (void) _handleMultiValueChanged: (NSNotification*) note;
@end

@implementation ADRecord(Private)
- (void) _handleMultiValueChanged: (NSNotification*) note
{
  ADMultiValue *object = [note object];
  if(![[_dict allValues] containsObject: object])
    return;
  NSString *prop; NSEnumerator *e = [[_dict allKeys] objectEnumerator];
  while((prop = [e nextObject]))
    if([_dict objectForKey: prop] == object)
      {
	[[NSNotificationCenter defaultCenter]
	  postNotificationName: ADRecordChangedNotification
	  object: self
	  userInfo: [NSDictionary dictionaryWithObjectsAndKeys:
				    object, ADChangedValueKey,
				  prop, ADChangedPropertyKey,
				  nil]];
	return;
      }
  // should never get here
  NSLog(@"NOT FOUND!\n");
}
@end

@implementation ADRecord
- init
{
  _dict = [[NSMutableDictionary alloc] initWithCapacity: 10];
  _book = nil;

  if([self isKindOfClass: [ADPerson class]])
    [self setValue: @"Person" forProperty: @"Type"];
  else if([self isKindOfClass: [ADGroup class]])
    [self setValue: @"Group" forProperty: @"Type"];

  [[NSNotificationCenter defaultCenter]
    addObserver: self
    selector: @selector(_handleMultiValueChanged:)
    name: @"_MVChg"
    object: nil];
  
  return [super init];
}

- (void) dealloc
{
  [[NSNotificationCenter defaultCenter] removeObserver: self];
  [_dict release];
  [super dealloc];
}

- (id) valueForProperty: (NSString *) property
{
  return [_dict objectForKey: property];
}

- (BOOL) setValue: (id) value
      forProperty: (NSString *) property
{
  if(!value || [value isEqual: @""])
    [_dict removeObjectForKey: property];
  else
    [_dict setObject: value forKey: property];

  if(![property isEqualToString: ADUIDProperty])
    [[NSNotificationCenter defaultCenter]
      postNotificationName: ADRecordChangedNotification
      object: self
      userInfo: [NSDictionary dictionaryWithObjectsAndKeys:
				value, ADChangedValueKey,
			      property, ADChangedPropertyKey,
			      nil]];
  
  return YES;
}

- (BOOL) removeValueForProperty: (NSString*) property
{
  [_dict removeObjectForKey: property];

  if(![property isEqualToString: ADUIDProperty])
    [[NSNotificationCenter defaultCenter]
      postNotificationName: ADRecordChangedNotification
      object: self
      userInfo: [NSDictionary dictionaryWithObjectsAndKeys:
				property, ADChangedPropertyKey,
			      nil]];
  return YES;
}

- (ADAddressBook*) addressBook
{
  return _book;
}

- (void) setAddressBook: (ADAddressBook*) book
{
  if(_book)
    [NSException raise: ADAddressBookConsistencyError
		 format: @"Cannot set address book on record '%@'"
		 @" (already has one)", [self uniqueId]];
  if(!book)
    [NSException raise: ADAddressBookConsistencyError
		 format: @"Cannot set nil address book on record '%@'",
		 [self uniqueId]];
  _book = book;
}
@end

@implementation ADRecord(ADRecord_Convenience)
- (NSString*) uniqueId
{
  return [self valueForProperty: ADUIDProperty];
}
@end

@implementation ADRecord(AddressesExtensions)
- (id) initWithRepresentation: (NSData*) data
			 type: (NSString*) type
{
  Class c = [self class];
  [self release];

  id<ADInputConverting> converter; id obj;

  converter = [[ADConverterManager sharedManager]
		inputConverterForType: type];
  if(!converter) return nil;

  if(![converter useData: data])
    return nil;

  obj = [converter nextRecord];
  if(!obj) return nil;

  if(![[obj class] isSubclassOfClass: c])
    {
      NSLog(@"It's of %@, not %@\n", [c className], [obj className]);
      return nil;
    }

  return [obj retain];
}

- (NSData*) representationWithType: (NSString*) type
{
  [self release];

  id<ADOutputConverting> converter; 

  converter = [[ADConverterManager sharedManager]
		outputConverterForType: type];
  [converter storeRecord: self];
  return [converter data];
}

- (NSString*) description
{
  return [_dict description];
}
@end
