// ADVCFConverter.m (this is -*- ObjC -*-)
// 
// \author: Bjrn Giesler <giesler@ira.uka.de>
// 
// Address Book Framework for GNUstep
// 
// $Author: giesler $
// $Locker:  $
// $Revision: 1.3 $
// $Date: 2002/08/19 08:41:28 $

/* system includes */
#include <AppKit/AppKit.h>

/* my includes */
#include "ADVCFConverter.h"
#include "ADPerson.h"
#include "ADGlobals.h"
#include "ADImageLoading.h"
#include "ADMultiValue.h"

@interface NSString(QuotedPrintable)
- (unsigned long) hexLongValue;
- (NSString*) stringByQuotedPrintableDecoding;
@end

@implementation NSString(QuotedPrintable)
- (unsigned long) hexLongValue
{
  unsigned long val = 0;
  NSString *str =
    [[self stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceCharacterSet]]
      lowercaseString];
  NSString *hexchars = @"0123456789abcdef";
  int i;

  for(i=0; i<[str length]; i++)
    {
      NSRange r = NSMakeRange(i, 1);
      r = [hexchars rangeOfString: [str substringWithRange: r]];
      assert(r.location != NSNotFound);
      val <<= 4;
      val |= r.location;
    }

  return val;
}

- (NSString*) stringByQuotedPrintableDecoding
{
  int i;
  
  NSMutableString *str = [NSMutableString stringWithCapacity: [self length]];
  for(i=0; i<[self length]; i++)
    {
      NSRange r = NSMakeRange(i, 1);
      NSString *s = [self substringWithRange: r];
      if([s isEqualToString: @"="] && i < [self length]-2)
	{
	  unsigned char c;
	  
	  r = NSMakeRange(i+1, 2);
	  NSString *hex = [self substringWithRange: r];
	  c = (unsigned char)[hex hexLongValue];

	  [str appendString: [NSString stringWithFormat: @"%c", c]];
	  i+=2;
	}
      else
	[str appendString: s];
    }

  str = [NSString stringWithUTF8String: [str cString]];
  return str;
}
@end

@interface NSArray(VCFKeys)
- (NSString*) restOfStringStartingWith: (NSString*) start;
@end

@implementation NSArray(VCFKeys)
- (NSString*) restOfStringStartingWith: (NSString*) start
{
  NSEnumerator *e = [self objectEnumerator];
  id obj;
  
  while((obj = [e nextObject]))
    {
      if(![obj isKindOfClass: [NSString class]])
	continue;
      if([obj length] < [start length])
	continue;
      if([[obj substringToIndex: [start length]]
	   isEqualToString: start])
	return [obj substringFromIndex: [start length]];
    }
  return nil;
}
@end

NSData *base64Decode(NSString* nsstr)
{
  char *b64chars =
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
  int i, j;
  unsigned char *bytes; const char *str;

  //assert((int)([nsstr length]/4) == ((float)[nsstr length])/4.0);
  str = [nsstr lossyCString];
  
  bytes = (unsigned char*)calloc(([nsstr length]/4)*3, 1);
  
  for(i=0, j=0; i < strlen(str); i+= 4, j+=3)
    {
      unsigned char s1, s2, s3, s4;
      u_int32_t v;

      while(!strchr(b64chars, str[i]) && i<strlen(str))
	i++;
      if(i>=strlen(str))
	{
	  NSLog(@"Error: No good characters in string\n");
	  break;
	}
      
      s1 = strchr(b64chars, str[i]) - b64chars;
      s2 = strchr(b64chars, str[i+1]) - b64chars;
      s3 = strchr(b64chars, str[i+2]) - b64chars;
      s4 = strchr(b64chars, str[i+3]) - b64chars;
      v = (s1 << 18) | (s2 << 12) | (s3 << 6) | s4;
      bytes[j] = (v & 0xff0000) >> 16;
      bytes[j+1] = (v & 0xff00) >> 8;
      bytes[j+2] = v & 0xff;
    }
  return [NSData dataWithBytes: bytes length: ([nsstr length]/4)*3];
}
      
@interface ADVCFConverter(Private)
- (BOOL) parseLine: (int) line
	 fromArray: (NSArray*) arr
	  upToLine: (int*) retLine
      intoKeyBlock: (NSArray**) k
	valueBlock: (NSArray**) v;
- (void) integrateKeyBlock: (NSArray*) k
		valueBlock: (NSArray*) v
		intoPerson: (ADPerson*) p;
@end

static NSArray *knownItems;

@implementation ADVCFConverter
+ (void) initialize
{
  knownItems =
    [[NSArray alloc] initWithObjects: @"org", @"role", @"url", @"adr",
		     @"n", @"agent", @"logo", @"photo", @"label", @"fn",
		     @"title", @"sound", @"version", @"tel", @"email",
		     @"tz", @"geo", @"note", @"bday", @"rev", @"uid",
		     @"key", @"mailer", nil];
}

- initForInput
{
  _input = YES;
  _str = nil;
  return [super init];
}

- (BOOL) useData: (NSData*) data
{
  char *c = calloc([data length]+1, 1);
  memcpy(c, [data bytes], [data length]);
  _str = [[NSString alloc] initWithCString: c];
  free(c);

  _idx = 0;

  return YES; 
}

- (ADRecord*) nextRecord
{
  ADPerson *person = [[[ADPerson alloc] init] autorelease];
  [person setValue: [NSDate date] forProperty: ADModificationDateProperty];
  [person setValue: [NSDate date] forProperty: ADCreationDateProperty];
  
  NSString *str = [_str substringFromIndex: _idx];
  
  NSArray *lines = [str componentsSeparatedByString: @"\n"];
  
  int i = 0;
  while(i < [lines count])
    {
      NSArray *keyblock, *valueblock;
      BOOL retval;
      int oldIndex, newIndex;

      oldIndex = i;
      retval = [self parseLine: i
		     fromArray: lines
		     upToLine: &i
		     intoKeyBlock: &keyblock
		     valueBlock: &valueblock];
      newIndex = i;
      while(oldIndex < newIndex)
	_idx += [[lines objectAtIndex: oldIndex++] length] + 1;
      
      if(retval)
	{
	  if([keyblock containsObject: @"end"]) // done!
	    return person;
	     
	  if(![keyblock containsObject: @"begin"] &&
	     ![keyblock containsObject: @"end"] &&
	     ![keyblock containsObject: @"version"])
	    [self integrateKeyBlock: keyblock
		  valueBlock: valueblock
		  intoPerson: person];
	  }
      }
  
  return nil;
}

/* COutputConverting */
- initForOutput
{
  [NSException raise: ADUnimplementedError
	       format: @"Converting to VCF not implemented"]; 
  return nil;
}

- (BOOL) canStoreMultipleRecords
{
  [NSException raise: ADUnimplementedError
	       format: @"Converting to VCF not implemented"];
  return NO;
}

- (void) storeRecord: (ADRecord*) record
{
  [NSException raise: ADUnimplementedError
	       format: @"Converting to VCF not implemented"];
}

- (NSData*) data
{
  [NSException raise: ADUnimplementedError
	       format: @"Converting to VCF not implemented"];
  return nil;
}
@end

@implementation ADVCFConverter (Private)
- (BOOL) parseLine: (int) line
	 fromArray: (NSArray*) arr
	  upToLine: (int*) retLine
      intoKeyBlock: (NSArray**) k
	valueBlock: (NSArray**) v
{
  NSString *str, *keyblock, *value;
  NSCharacterSet *wsp = [NSCharacterSet whitespaceAndNewlineCharacterSet];
  
  *retLine = line;
  
  str = [[arr objectAtIndex: (*retLine)++]
	    stringByTrimmingCharactersInSet: wsp];
  if(![str length]) return NO;
  while(*retLine < [arr count] &&
	[[arr objectAtIndex: *retLine] length] &&
	[[[arr objectAtIndex: *retLine]
	   substringWithRange: NSMakeRange(0, 1)]
	  isEqualToString: @" "])
    {
      NSString *str2 = [[arr objectAtIndex: (*retLine)++]
			 stringByTrimmingCharactersInSet: wsp];
      str = [str stringByAppendingString: @" "];
      str = [str stringByAppendingString: str2];
    }
		 
  NSRange r = [str rangeOfString: @":"];
  if(r.location == NSNotFound)
    {
      NSLog(@"Syntax error in line %d!\n", line);
      return NO;
    }
  
  keyblock = [str substringToIndex: r.location];
  *k = [[keyblock lowercaseString] componentsSeparatedByString: @";"];
  
  value = [str substringFromIndex: r.location+r.length];
  if([value isEqualToString: @"="]) // escape to next line
    {
      value = [[arr objectAtIndex: (*retLine)++]
		stringByTrimmingCharactersInSet: wsp];
      *v = [[value stringByQuotedPrintableDecoding]
	     componentsSeparatedByString: @";"];
    }
  else
    *v = [[value stringByQuotedPrintableDecoding]
	   componentsSeparatedByString: @";"];

  return YES;
}

- (void) integrateKeyBlock: (NSArray*) k
		valueBlock: (NSArray*) v
		intoPerson: (ADPerson*) p
{
  NSString* key;

  key = [k objectAtIndex: 0];
  if(![knownItems containsObject: key])
    {
      NSLog(@"%@ not in knownItems\n", key);
      return;
    }

  if([key isEqualToString: @"n"])
    {
      [p setValue: [v objectAtIndex: 0] forProperty: ADLastNameProperty];
      [p setValue: [v objectAtIndex: 1] forProperty: ADFirstNameProperty];
      [p setValue: [v objectAtIndex: 2] forProperty: ADMiddleNameProperty];
      [p setValue: [v objectAtIndex: 3] forProperty: ADTitleProperty];
      [p setValue: [v objectAtIndex: 4] forProperty: ADSuffixProperty];
    }

  else if([key isEqualToString: @"org"])
    [p setValue: [v objectAtIndex: 0] forProperty: ADOrganizationProperty];
  else if([key isEqualToString: @"title"])
    [p setValue: [v objectAtIndex: 0] forProperty: ADJobTitleProperty];
  else if([key isEqualToString: @"url"])
    [p setValue: [v objectAtIndex: 0] forProperty: ADHomePageProperty];

  else if([key isEqualToString: @"bday"])
    [p setValue: [NSDate dateWithString: [v objectAtIndex: 0]]
       forProperty: ADBirthdayProperty];
  
  else if([key isEqualToString: @"note"])
    [p setValue: [v objectAtIndex: 0] forProperty: ADNoteProperty];

  // phone -- multi-string
  else if([key isEqualToString: @"tel"])
    {
      ADMutableMultiValue *mv = [p valueForProperty: ADPhoneProperty];
      NSString *val = [v objectAtIndex: 0];

      if([k containsObject: @"fax"])
	{
	  if([k containsObject: @"home"])
	    [mv addValue: val withLabel: ADPhoneHomeFAXLabel];
	  else
	    [mv addValue: val withLabel: ADPhoneWorkFAXLabel];
	}
      else if([k containsObject: @"pager"])
	{
	  [mv addValue: val withLabel: ADPhonePagerLabel];
	}
      else // assume "voice" for everything else
	{
	  if([k containsObject: @"main"])
	    [mv addValue: val withLabel: ADPhoneMainLabel];
	  else if([k containsObject: @"cell"])
	    [mv addValue: val withLabel: ADPhoneMobileLabel];
	  else if([k containsObject: @"home"])
	    [mv addValue: val withLabel: ADPhoneHomeLabel];
	  else
	    [mv addValue: val withLabel: ADPhoneWorkLabel];
	}
      [p setValue: mv forProperty: ADPhoneProperty];
    }

  // email -- multi-string
  else if([key isEqualToString: @"email"])
    {
      ADMutableMultiValue *mv = [p valueForProperty: ADEmailProperty];

      if([k containsObject: @"home"])
	[mv addValue: [v objectAtIndex: 0] withLabel: ADEmailHomeLabel];
      else
	[mv addValue: [v objectAtIndex: 0] withLabel: ADEmailWorkLabel];
    }

  else if([key isEqualToString: @"adr"])
    {
      ADMutableMultiValue *mv = [p valueForProperty: ADAddressProperty];

      NSMutableDictionary *dict =
	[NSMutableDictionary dictionaryWithCapacity: 6];
      NSString *poBox        = [v objectAtIndex: 0];
      NSString *extendedAddr = [v objectAtIndex: 1];
      NSString *street       = [v objectAtIndex: 2];
      NSString *locality     = [v objectAtIndex: 3];
      NSString *region       = [v objectAtIndex: 4];
      NSString *postalCode   = [v objectAtIndex: 5];
      NSString *countryName  = [v objectAtIndex: 6];

      if(street && ![street isEqualToString: @""])
	[dict setObject: street forKey: ADAddressStreetKey];
      if(locality && ![locality isEqualToString: @""])
	[dict setObject: locality forKey: ADAddressCityKey];
      if(region && ![region isEqualToString: @""])
	[dict setObject: region forKey: ADAddressStateKey];
      if(postalCode && ![postalCode isEqualToString: @""])
	[dict setObject: postalCode forKey: ADAddressZIPKey];
      if(countryName && ![countryName isEqualToString: @""])
	[dict setObject: countryName forKey: ADAddressCountryKey];
      if(poBox && ![poBox isEqualToString: @""])
	[dict setObject: poBox forKey: ADAddressPOBoxKey];
      if(extendedAddr && ![extendedAddr isEqualToString: @""])
	[dict setObject: extendedAddr forKey: ADAddressExtendedAddressKey];
      
      if([k containsObject: @"home"])
	[mv addValue: dict withLabel: ADAddressHomeLabel];
      else
	[mv addValue: dict withLabel: ADAddressWorkLabel];
    }

  else if([key isEqualToString: @"photo"])
    {
      NSString *encoding = [k restOfStringStartingWith: @"encoding="];
      if(![encoding isEqualToString: @"base64"])
	{
	  NSLog(@"Cannot integrate image -- unknown "
		@"encoding '%@'\n", encoding);
	  return;
	}
      NSString *type = [k restOfStringStartingWith: @"type="];

      NSData *data = base64Decode([v objectAtIndex: 0]);

      // Let's hope NSImage handles this
      [p setImageData: data];
      if(type)
	[p setImageDataType: type];
    }

  // FIXME: The following keys (specified in the vcard spec) aren't
  // handled by this converter yet, because they're not in the Apple
  // spec, but should perhaps be: 
  // @"label" -- free-form postal delivery label text
  // @"key"   -- public key
  // @"rev"   -- last revision in ISO8601 format, which NSDate can't
  //             parse (yet)
  else
    NSLog(@"Warning: Unhandled key '%@' in vcard file\n", key);
}
@end
