#!/bin/sh

SUBDIRS="AddressesFramework AddressViewFramework addresstool AddressManager"
FORCE=no

while [ $# != 0 ]; do
	case $1 in
	--force)
	        echo "Force set to yes."
		FORCE=yes
		shift
		;;
	--target)
	        shift
		TARGET=$1
		shift
		;;
	*)
		echo "Unknown parameter $1"
		echo "Usage: $0 [--force] [--target TGT]"
		echo -n "    --force: Force trying to build and install, "
		echo "even if not root"
		echo "    --target TGT: make TGT instead of all"
		exit
		;;
	esac
done
		

if [ $UID != 0 -a $FORCE != yes ]; then
	echo "Must be root to run this script (or use --force)"
	exit
fi

for d in $SUBDIRS; do
    echo "Building in $d..."
    make -C $d $TARGET
    if [ $? != 0 ]; then
        cat <<EOF

*****
***** Error compiling $d!
***** Please contact Bjrn Giesler <bjoern@giesler.de> and report this error!
***** Please include the error messages leading up to this in your mail.
***** Thank you.
*****

EOF
        exit
    fi

    if [ x$TARGET != x ]; then
	continue
    fi

    make -C $d install
    if [ $? != 0 ]; then
	cat <<EOF

*****
***** Couldn't install $d -- insufficient permissions? Try as root!
*****

EOF
	if [ $FORCE != yes ]; then
	    exit
	else
	    cat <<EOF

*****
***** Continuing because of --force...
*****

EOF
        fi
    fi
done

